/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Media Inspector

var nt = (function() {

  var EMPTY_IMAGE = 'data:image/gif;base64,R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7';

  var fadeSpeed = is.FADE_SPEED;

  var dcount = 0;
  var darray = [];

//  var retina = (window.deviceScale)

  var busy = false;
  var tr = is.tr;
  var id = rti.did;
  var ssdpURL = about.rest + '/ssdp';

  function spin() {
    busy = true;
    $('.onbusy').disable();
    is.spin();
  }

  function nospin() {
    busy = false;
    $('.onbusy').enable();
    is.nospin();
  }

  function hideWaiting() {
    $('#nodevs').fadeOut(fadeSpeed);
  }

  function showWaiting() {
    if (!busy && dcount === 0) {
      $('#nodevs').fadeIn(fadeSpeed);
      $('#dlist').toggleClass('btop', false);
    }
  }

  function dname(dev) {
    var dcfg = dev[id.dcfg];
    return dcfg[id.alias] || dev[id.friendlyName] || dev[id.addr];
  }

  function latest(dev) {
    var uuid = dev[id.uuid];
    for (var i = 0; i < darray.length; i++) {
      var last = darray[i];
      if (last[id.uuid] === uuid) {
//        console.log('Latest dev found: ' + dname(dev));
        return last;
      }
    }
    return dev;
  }

  //-----------------------------------------------------------------------------
  // Actions
  //-----------------------------------------------------------------------------

  function actAjax(path, args, done, fail) {
    $.ajax({
        url: ssdpURL + path,
        data: args,
        cache: false
      }
    )
      .done(done)
      .fail(fail);
  }

  function actBlock(hash, dev, btn) {
    if (!confirm(tr('Are you sure you want to block all devices from') + ': ' + dev[id.addr] + '?')) return;
    btn.disable();
    actAjax(
      '/block',
      {uuid: dev[id.uuid]},
      function done() {
        $('#' + hash).remove();
        dcount--;
        showWaiting();
      },
      function fail(xhr) {
        btn.enable();
        is.showError(is.xhrerror(xhr), 5000);
      }
    );
  }

  function actHide(hash, dev, btn) {
    btn.disable();
    actAjax(
      '/hide',
      {uuid: dev[id.uuid]},
      function done() {
        $('#' + hash).remove();
        dcount--;
        showWaiting();
      },
      function fail(xhr) {
        btn.enable();
        is.showError(is.xhrerror(xhr), 5000);
      }
    );
  }

  function actRename(hash, dev, btn) {
    var name = prompt(tr('Rename device') + ':', $('#' + hash).find('.dname').text());
    if (name === null) return;
    name = name.trim();
    btn.disable();
    actAjax(
      '/rename',
      {uuid: dev[id.uuid], name: name},
      function done() {
        dev[id.dcfg][id.alias] = (name.length > 0) ? name : null;
        $('#' + hash).find('.dname').text(dname(dev));
        btn.enable();
      },
      function fail(xhr) {
        btn.enable();
        is.showError(is.xhrerror(xhr), 5000);
      }
    );
  }

  function actReport(hash, dev, btn) {
    var last = JSON.parse(JSON.stringify(latest(dev)));
    delete last[id.ctime];
    delete last[id.mtime];
    delete last[id.atime];
    delete last[id.hash];
    var data = JSON.stringify(last, null, 2);
    data = data.replace(/"/g, '');
    is.devReport(data);
  }

  //-----------------------------------------------------------------------------
  // Update
  //-----------------------------------------------------------------------------

  function menuItem(hash, dev, btn, label, click) {
    var l = $('<li/>');
    var a = $('<a href="#"/>');
    a.text(label);
    a.click(function(evt) {
      evt.preventDefault();
      setTimeout(function() {
        click(hash, dev, btn);
      }, 10);
    });
    l.append(a);
    return l;
  }

  function menuSeparator() {
    return $('<li class="divider"></li>');
  }

  function createButton(hash, dev) {

    var div = $('<div class="btn-group"/>');

    var btn = $('<button type="button" class="btn btn-default btn-sm dropdown-toggle" data-toggle="dropdown"/>');

    div.append(btn);

    btn.append('<span class="glyphicon glyphicon-cog"></span>');

    var mnu = $('<ul class="dropdown-menu pull-right" role="menu"/>');

    div.append(mnu);

    mnu.append(menuItem(hash, dev, btn, tr('Block') + '...', actBlock));
    mnu.append(menuItem(hash, dev, btn, tr('Hide'), actHide));
    mnu.append(menuSeparator());

    mnu.append(menuItem(hash, dev, btn, tr('Rename') + '...', actRename));
    mnu.append(menuSeparator());

    mnu.append(menuItem(hash, dev, btn, tr('Send Report'), actReport));

    return div;

  }

  function createIcon(hash, dev) {
    var icons = dev[id.icons];
    if (icons) {
      var jicons = icons['image/jpeg'];
      if (jicons) {
        var sz48 = jicons['48'];
        if (sz48) {
          var img = $('<img width="48" height="48"/>');
          img.attr('src', sz48['u']);
          return img;
        }
      }
    }
    return $('<span class="glyphicon glyphicon-link"></span>');
  }

  function createDevice(hash, dev) {

    var tb = $('<table class="dev"/>');
    tb.attr('id', hash);
    var tr = $('<tr/>');

    var c1 = $('<td class="c1"/>');
    c1.append(createIcon(hash, dev));

    var c2 = $('<td class="c2"/>');

    var nl = $('<a href="#" class="dname"/>');

    nl.click(function(evt) {
      evt.preventDefault();
      if (is.isMetaDown()) {
        return actReport(hash, dev);
      }
      var uuid = dev[id.uuid];
      is.host.openWin(uuid, about.face + '/dconf/' + uuid, 'Loading...', 500, 600);
    });

    nl.text(dname(dev));

    c2.append(nl);

    if (dev[id.manufacturer]) {
      c2.append('<br>');
      c2.append(dev[id.manufacturer]);
    }

    var c3 = $('<td class="c3"/>');
    c3.append(createButton(hash, dev));

    tr.append(c1).append(c2).append(c3);

    tb.append(tr);

    return tb;

  }

  function updateDevice(hash, dev, tbl) {
    $('#' + hash).find('.dname').text(dname(dev));
  }

  function update(dary) {
    var lst = $('#dlist');
    var len = dary.length;
    dcount = len;
    darray = dary;
    if (len === 0) {
      lst.empty();
      return;
    }
    hideWaiting();
    lst.toggleClass('btop', true);
    var found = {};
    for (var i = 0; i < len; i++) {
      var dev  = dary[i];
      var hash = dev[id.hash];
      found[hash] = true;
      var tbl  = $('#' + hash);
      if (tbl.length > 0) {
        updateDevice(hash, dev, tbl);
      }
      else {
        lst.append(createDevice(hash, dev));
      }
    }
    $('.dev').each(function() {
      if (!found[this.id]) {
        $(this).remove();
      }
    });
  }

  //-----------------------------------------------------------------------------
  // Search
  //-----------------------------------------------------------------------------

  function search() {
    if (busy) return;
    hideWaiting();
    spin();
    $.ajax({
        url: ssdpURL + '/search',
        cache: false
      }
    )
      .done(function(devs) {
        nospin();
        update(devs);
        showWaiting();
      })
      .fail(function(xhr) {
        nospin();
        is.showError(is.xhrerror(xhr), 5000);
        showWaiting();
      });
  }



  // Key Binding

//  $(document).keydown(function(evt) {
//    if (!busy && evt.keyCode == 79) {
//      chooseFile(mode);
//      evt.preventDefault();
//      evt.stopPropagation();
//    }
//  });

  //-----------------------------------------------------------------------------
  // Refresh
  //-----------------------------------------------------------------------------

  function refresh() {
    $.ajax({
        url: ssdpURL + '/devs',
        cache: false
      }
    )
      .done(function(devs) {
        update(devs);
        showWaiting();
        setTimeout(refresh, 2000);
      })
      .fail(function(xhr) {
        is.showError(is.xhrerror(xhr), 5000);
        showWaiting();
      });
  }

  $(document).ready(refresh);

  //-----------------------------------------------------------------------------

  function refreshFull() {
    if (busy) return;
    hideWaiting();
    spin();
    $.ajax({
        url: ssdpURL + '/refresh',
        cache: false
      }
    )
      .done(function(devs) {
        nospin();
        update(devs);
        showWaiting();
      })
      .fail(function(xhr) {
        nospin();
        is.showError(is.xhrerror(xhr), 5000);
        showWaiting();
      });
  }


  //-----------------------------------------------------------------------------

  function notifyAll() {
    if (busy) return;
    hideWaiting();
    spin();
    $.ajax({
        url: ssdpURL + '/notify',
        cache: false
      }
    )
      .done(function() {
        nospin();
        showWaiting();
        is.showInfo(is.tr('Notification has been sent'), 3000);
      })
      .fail(function(xhr) {
        nospin();
        is.showError(is.xhrerror(xhr), 5000);
        showWaiting();
      });
  }

  //-----------------------------------------------------------------------------
  // Public Interface
  //-----------------------------------------------------------------------------

  return {
    search: search,
    refresh: refreshFull,
    notifyAll: notifyAll
  };



})();