/*
 * Copyright (c) 2015 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var spawn  = require('child_process').spawn;
var byline = require('byline');

var env = require('../env');
var log = require('../log');

//-----------------------------------------------------------------------------

var GLOBAL_ARGS = [
  '-hide_banner',
  '-nostats',
  '-y'
];

//-----------------------------------------------------------------------------

function Coder(csn, opt) {

  var self  = this;
  var islog = log.canCoder();
  var ended = false;
  var res   = opt.res;

  var pcoder = null;
  var stdout = null;
  var stderr = null;
  var exited = false;

  //-----------------------------------------------------------------------------

  function onSocketClose() {
    if (islog) {
      log.coder('[%s] socket closed', csn);
    }
    resEnd();
    stop();
  }

  function onResFinish() {
    ended = true;
    if (islog) {
      log.coder('[%s] response finished', csn);
    }
    if (opt.ecb) {
      setImmediate(opt.ecb);
    }
  }

  function resEnd() {
    if (!ended) {
      ended = true;
      if (stdout) {
        stdout.unpipe(res);
      }
      res.end();
    }
  }

  function bind() {
    res.once('finish', onResFinish);
    res.once('close', onSocketClose);
  }

  function unbind() {
    res.removeListener('finish', onResFinish);
    res.removeListener('close', onSocketClose);
    if (pcoder) {
      pcoder.removeAllListeners();
      stderr.removeListener('data', onStdErr);
      pcoder.stdout.unpipe();
      pcoder.stderr.unpipe();
      pcoder.stdout.removeAllListeners();
      pcoder.stderr.removeAllListeners();
    }
  }

  //-----------------------------------------------------------------------------

  function launch(arg) {
    if (islog) {
      log.coder('[%s] %s', csn, arg.join(' '));
    }
    var all = GLOBAL_ARGS.concat('-loglevel', islog ? 'error' : 'fatal', arg);
    pcoder = spawn(opt.bin, all, {env: env.coderEnv, detached: false, stdio: ['pipe', 'pipe', 'pipe']});
    pcoder.once('exit', onCoderExit);
    pcoder.once('error', onCoderError);
    stdout = pcoder.stdout;
    pcoder.stderr.setEncoding('utf8');
    stderr = byline.createStream(pcoder.stderr);
    stderr.on('data', onStdErr);
    stdout.pipe(res);
  }

  function start() {
    bind();
    opt.buildArgs(function(err, arg) {
      if (ended) {
        unbind();
        return;
      }
      launch(arg);
    });
  }

  function stop() {
    if (pcoder && !exited) {
      pcoder.stdin.end();
      stdout.on('data', dummy);
      pcoder.kill('SIGKILL');
    }
  }

  //-----------------------------------------------------------------------------

  function onCoderExit() {
    exited = true;
    log.coder('[%s] coder exit', csn);
    resEnd();
    unbind();
  }

  //-----------------------------------------------------------------------------

  function onCoderError(err) {
    exited = true;
    log.error('[%s] %s', csn, err);
    resEnd();
    unbind();
  }

  //-----------------------------------------------------------------------------

  function onStdErr(line) {
    if (islog) {
      log.coder('[%s] %s', csn, line);
    }
    else {
      log.error('[%s] %s', csn, line);
    }
  }

  //-----------------------------------------------------------------------------

  self.start = start;

}

exports.Coder = Coder;

//-----------------------------------------------------------------------------

function dummy(data) {}