/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Media Database

'use strict';

var fs  = require('fs');
var np  = require('path');

var evt = require("../evt");
var cfg = require("../cfg");
var rti = require("../rti");
var log = require("../log");
var fop = require("../fop");

var pkg = require(global['ISEDORA_PKG']);

var cache = require('../cls/cache');

var DEF_CACHE_VER = 200;
var CACHE_VER_FN  = 'version.txt';

var THUMB_EXT = '.jpg';
var SCALE_EXT = '.jpg';
var SRT_EXT   = '.srt';

var thumbCache = new cache.SizeCache('thumb', THUMB_EXT, 32);
var scaleCache = new cache.SizeCache('scale', SCALE_EXT, 32);
var vtextCache = new cache.SizeCache('vtext', SRT_EXT, 32);
var minfoCache = new cache.JsonFileCache('minfo', '.json', 32);
var parents    = new cache.KeyValStore('parents', 3000, true);

exports.thumb   = thumbCache;
exports.minfo   = minfoCache;
exports.scale   = scaleCache;
exports.vtext   = vtextCache;
exports.parents = parents;

//-----------------------------------------------------------------------------

function loadCaches(erase) {
  thumbCache.load(cfg.thumbCache(), erase);
  scaleCache.load(cfg.scaleCache(), erase);
  minfoCache.load(cfg.minfoCache(), erase);
  vtextCache.load(cfg.vtextCache(), false);
  parents.load(cfg.cacheDir(), 'parents.json', erase);
}

//-----------------------------------------------------------------------------

function onConfigReady() {

  var cacheVersion = pkg['cacheVersion'] || DEF_CACHE_VER;

  var vfile = np.join(cfg.cacheDir(), CACHE_VER_FN);

  fs.readFile(vfile, fop.FREAD_OPT(), function(err, text) {

    var vsave = false;
    var erase = false;

    if (err) {
      vsave = true;
      erase = true;
    }
    else {
      var currentVersion = parseInt(text, 10);
      if (isNaN(currentVersion) || (cacheVersion > currentVersion)) {
        vsave = true;
        erase = true;
      }
    }

    loadCaches(erase);

    if (erase) {
      evt.postUpdateCache();
    }

    if (vsave) {
      fs.writeFile(vfile, String(cacheVersion), fop.FWRITE_OPT());
    }

  });

}

evt.onConfigReady(onConfigReady);



