/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// JavaScript Language Helpers

'use strict';

exports.overload = function(self, name, proc) {
  var parent = self[name];
  self[name] = function() {
    if (parent) parent.apply(self, arguments);
    proc.apply(self, arguments);
  };
};

//-----------------------------------------------------------------------------

function hasPrefix(str, prefix) {
  if (typeof str !== 'string' || typeof  prefix !== 'string') {
    return false;
  }
  var len = prefix.length;
  if (len > str.length || len === 0) {
    return false;
  }
  for (var i = 0; i < len; i++) {
    if (str.charAt(i) !== prefix.charAt(i)) {
      return false;
    }
  }
  return true;
}

exports.hasPrefix = hasPrefix;

//-----------------------------------------------------------------------------

function hasSuffix(str, suffix) {
  if (typeof str !== 'string' || typeof  suffix !== 'string') {
    return false;
  }
  var xlen = suffix.length;
  var slen = str.length;
  if (xlen > slen || xlen === 0) {
    return false;
  }
  return (str.indexOf(suffix, slen - xlen) !== -1);
}

exports.hasSuffix = hasSuffix;

//-----------------------------------------------------------------------------

exports.getArrayPrefixIndex = function(ary, fullString) {
  var rv = [];
  for (var i = 0; i < ary.length; i++) {
    if (hasPrefix(fullString, ary[i])) {
      rv.push(i);
    }
  }
  var len = rv.length;
  if (len === 0) return -1;
  if (len === 1) return rv[0];
  rv.sort(function(ix1, ix2) {
    return (ary[ix2].length - ary[ix1].length);
  });
  return rv[0];
};

//-----------------------------------------------------------------------------

function copyValue(val) {
  if (val === undefined || val === null) {
    return val;
  }
  var tof = typeof val;
  if (tof === 'number' || tof === 'string' || tof === 'boolean' || tof === 'function') {
    return val;
  }
  if (Array.isArray(val)) {
    var ary = [];
    var len = val.length;
    for (var i = 0; i < len; i++) {
      ary.push(copyValue(val[i]));
    }
    return ary;
  }
  if (val instanceof Date) {
    return new Date(val.getTime());
  }
  var obj = {};
  for (var key in val) {
    obj[key] = copyValue(val[key]);
  }
  return obj;
}

exports.copyValue = copyValue;

function mergeObject(src, dst) {
  if (!src || !dst) return;
  for (var key in src) {
    if (dst[key] === undefined) {
      dst[key] = copyValue(src[key]);
    }
  }
}

exports.mergeObject = mergeObject;

//-----------------------------------------------------------------------------

function allkeys(obj) {

  var ary = [];

  for (var key in obj) {
    ary.push(key);
  }

  return ary;

}

exports.allkeys = allkeys;

//-----------------------------------------------------------------------------

function aryAddStr(dst, val, sep) {
  if (val === null || val === undefined) {
    return;
  }
  val = val.trim();
  if (val.length === 0) {
    return;
  }
  var ary = val.split(sep);
  var len = ary.length;
  for (var i = 0; i < len; i++) {
    var str = ary[i].trim();
    if (str.length > 0) {
      dst.push(str);
    }
  }
}

exports.aryAddStr = aryAddStr;