/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var prf = require('../dlna/prf');
var hnd = require('../hnd');
var ffm = require('../ffm');
var rti = require('../rti');
var cfg = require('../cfg');


var id = rti.id;

var LPCM_FULL  = 'lpcm-full';
var LPCM_SHORT = 'lpcm-short';

var A44 = 44100;
var A48 = 48000;

function lpcm(dev, opt, full) {

  var aud = opt.aud;

  var tasr = aud[id.samplingrate] || A44;

  var dasr = dev.asr;

  if (tasr > dasr) {
    tasr = dasr;
  }
  else if (tasr < A44) {
    tasr = A44;
  }

  var pmime = prf.mime.lpcm;

  if (full) {
    pmime += ';rate=' + tasr + ';channels=2';
  }

  opt.pro = prf.newConvProfile('LPCM', pmime, opt.len);

  // Transcoding

  opt.acc = ffm.codec.pcm;
  opt.chn = 2;
  opt.asr = tasr;
  opt.mux = ffm.mux.aiff;

}

//-----------------------------------------------------------------------------
// Handler Hooks
//-----------------------------------------------------------------------------

function lpcmFull(dev, opt) {
  return lpcm(dev, opt, true);
}

function lpcmShort(dev, opt) {
  return lpcm(dev, opt, false);
}

hnd.addTcoder(LPCM_FULL, lpcmFull);
hnd.addTcoder(LPCM_SHORT, lpcmShort);