/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var hnd = require('../hnd');
var rti = require('../rti');

var id = rti.id;

var IX_KIND    = 0;
var IX_FORMAT  = 1;
var IX_AUDIO   = 2;
var IX_VIDEO   = 3;
var IX_PROFILE = 4;

var A44 = 44100;
var A48 = 48000;

var ABR_MULTI  = 448 * 1000;
var ABR_STEREO = 256 * 1000;

//-----------------------------------------------------------------------------
// Native formats analyzer
//-----------------------------------------------------------------------------

function findCodec(codecs, tracks) {

  if (codecs.length === 0) {
    return true;
  }

  if (!Array.isArray(tracks)) {
    return false;
  }

  var len = tracks.length;

  for (var i = 0; i < len; i++) {
    var trk = tracks[i];
    var cid = trk[id.codec];
    if (cid !== undefined && cid !== null && codecs.indexOf(cid) > -1) {
      return true;
    }
  }

  return false;

}

// ['a', 'mpeg audio', ['audio codecs'], ['video codecs], ['profile name', 'mime type]]

function analyzeNative(ary, md) {
  if (ary[IX_FORMAT] === '*') {
    return true;
  }
  if (ary[IX_FORMAT] === rti.codec(md)) {
    switch (ary.length) {
      case 2:
        return true;
      case 3:
        return findCodec(ary[IX_AUDIO], md[id.audio]);
      case 4:
      case 5:
        return findCodec(ary[IX_AUDIO], md[id.audio]) &&
               findCodec(ary[IX_VIDEO], md[id.video]);
    }

  }
  return false;
}

function isNative(dev, md) {

  var avt = rti.isAudio(md) ? 'a' : 'v';
  var fmt = dev.fmt;
  var len = fmt.length;

  if (len === 0) {
    return false;
  }

  for (var i = 0; i < len; i++) {
    var ary = fmt[i];
    if ((ary[IX_KIND] === avt) && analyzeNative(ary, md)) {
      return true;
    }
  }

  return false;

}

exports.isNative = isNative;

//-----------------------------------------------------------------------------
// Coder Helpers
//-----------------------------------------------------------------------------

exports.audioSampleRate = function(dev, aud) {

  var asr = aud[id.samplingrate] || A44;

  var dsr = dev.asr || A48;

  if (asr > dsr) {
    asr = dsr;
  }
  else if (asr < A44) {
    asr = A44;
  }

  return asr;

};

exports.audioBitRate = function(chn) {
  return (chn > 3) ? ABR_MULTI : ABR_STEREO;
};