/*
 * Copyright (c) 2015 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Fontconfig cache management

'use strict';

var np = require('path');
var fs = require('fs');
var execFile = require('child_process').execFile;

var evt = require('../evt');
var cfg = require('../cfg');
var env = require('../env');
var log = require('../log');
var ffm = require('../ffm');

// 698e8de9c79e614b8d6a96b5ce9682e6-le64.cache-5

var CACHE_SUFFIX = /^\w{32}-\w+\.cache-\d$/;
var MEDIA_FILE   = 'fancy.mkv';
var WINTEMP      = '\\Local Settings\\Temp';
var CFG_FILE     = 'fonts.xml';

var cacheFilePath;
var updateCache = false;

//-----------------------------------------------------------------------------

function rebuildFontCache() {

  log.debug('Building font cache...');

  var start = Date.now();

  var args = ['-hide_banner', '-y', '-loglevel', 'error'];

  var mfile = np.join(env.resourceDir, MEDIA_FILE);

  args.push('-i', mfile);

  args.push('-vf', "subtitles=filename=" + ffm.filterEscape(mfile));

  args.push('-f', 'vob', env.nullDevice);

  var bin = cfg.getEncBin(env.coderPath);

  execFile(bin, args, {env: env.coderEnv}, function(error, stdout, stderr) {

    if (error) {
      if (typeof stderr === 'string') stderr = stderr.trim();
      log.error('Error building font cache:\n%s', stderr);
      return;
    }

    log.debug('Font cache built in: %s sec', log.millisPeriod(start));

  });

}

//-----------------------------------------------------------------------------

function checkFontCache() {

  var cdir;

  if (env.isMac()) {
    cdir = cfg.cacheDir();
  }
  else if (env.isWin()) {
    cdir = process.env['USERPROFILE'] + WINTEMP;
  }
  else {
    throw new Error('Unsupported runtime environment');
  }

  log.debug('Checking font cache at: %s', cdir);

  fs.readdir(cdir, function(err, names) {

    if (err) {
      log.error('Error reading fontconfig cache dir: %s (%s)', cdir, err);
      return;
    }

    var len = names.length;
    var found = false;

    for (var i = 0; i < len; i++) {
      if (CACHE_SUFFIX.test(names[i])) {
        found = true;
        cacheFilePath = np.join(cdir, names[i]);
        if (updateCache) {
          updateCache = false;
          fs.unlink(cacheFilePath, function() {
            rebuildFontCache();
          });
        }
        break;
      }
    }

    if (!found) {
      rebuildFontCache();
    }

  });

}

//-----------------------------------------------------------------------------

function onConfigReady() {

  env.coderEnv['FC_CONFIG_DIR']   = env.resourceDir;
  env.coderEnv['FC_CONFIG_FILE']  = CFG_FILE;

  env.coderEnv['FONTCONFIG_PATH'] = env.resourceDir;
  env.coderEnv['FONTCONFIG_FILE'] = CFG_FILE;

  env.coderEnv['LC_CTYPE'] = 'en_US.UTF-8';

  setTimeout(checkFontCache, 1000); // be nice while starting

}

function onUpdateCache() {
  updateCache = true;
}

evt.onConfigReady(onConfigReady);
evt.onUpdateCache(onUpdateCache);
