/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var fs    = require('fs');
var np    = require('path');
var async = require('async');

var evt = require('../evt');
var agt = require('../agt');
var cfg = require('../cfg');

//-----------------------------------------------------------------------------

function parseAV(fmap, data, cb) {

  var video = [];
  var audio = [];
  var text  = [];

  fmap.video = video;
  fmap.audio = audio;
  fmap.text  = text;

  var lines = data.split('\n');
  var len = lines.length;

  for (var i = 0; i < len; i++) {

    var str = lines[i].trim();

    if (str.length === 0) continue;

    var avt = str.charAt(0);

    var ary;

    if      (avt === 'V') ary = video;
    else if (avt === 'A') ary = audio;
    else if (avt === 'S') ary = text;
    else continue;

    var pos = str.indexOf(' ', 2);

    if (pos < 3) continue;

    var name = str.substring(2, pos);
    var desc = str.substring(pos + 1);

    ary.push([desc, name]);

  }

  cb();

}

//-----------------------------------------------------------------------------

function readAV(fmap, cb) {

  var fpath = np.join(cfg.pubDir(), 'formats.txt');

  fs.readFile(fpath, {encoding: 'utf8'}, function(err, data) {
    if (err) {
      cb();
    }
    else {
      parseAV(fmap, data, cb);
    }
  });

}

function readImage(fmap, cb) {

  agt.formats(function(err, data) {
    if (!err) {
      fmap.image = data.image;
    }
    cb();
  });

}

//-----------------------------------------------------------------------------

function getFormats(req, res) {

  var fmap = {};

  async.parallel([
      function(cb) { readAV(fmap, cb) },
      function(cb) { readImage(fmap, cb) }
    ],
    function() {
      res.json(fmap);
    }
  );

}

evt.onHttpAppReady(function(app) {
  cfg.rest(app).get('/formats', getFormats);
});
