/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var evt = require('../evt');
var cfg = require('../cfg');
var rti = require('../rti');
var dev = require('../ssdp/dev');
var srv = require('../dms1/media_server_1').srv;
var search = require('../ssdp/search');
var bodyParser = require('body-parser');

var id = rti.did;

//-----------------------------------------------------------------------------

function onSearch(req, res) {
  search.search(function(err) {
    if (err) {
      res.status(500).send(err.toString());
    }
    else {
      res.json(dev.getDevices());
    }
  });
}

//-----------------------------------------------------------------------------

function onDevs(req, res) {
  res.json(dev.getDevices());
}

//-----------------------------------------------------------------------------

function onBlock(req, res) {
  var uuid = req.param('uuid');
  if (!uuid) {
    return res.status(400).send('Missing address parameter');
  }
  var d = dev.getDevice(uuid);
  if (!d) {
    return res.status(404).send('Device not found');
  }
  cfg.addBlockedIP(d[id.addr]);
  saveConfig(res);
}

//-----------------------------------------------------------------------------

function onHide(req, res) {
  var uuid = req.param('uuid');
  if (!uuid) {
    return res.status(400).send('Missing address parameter');
  }
  if (!dev.hasDevice(uuid)) {
    return res.status(404).send('Device not found');
  }
  cfg.setDevHidden(uuid, true);
  saveConfig(res);
}

//-----------------------------------------------------------------------------

function onRename(req, res) {
  var uuid = req.param('uuid');
  var name = req.param('name');
  if (!uuid) {
    return res.status(400).send('Missing rename parameters');
  }
  var dcfg = cfg.getDevCfg(uuid);
  if (!dcfg) {
    return res.status(404).send('Device config not found');
  }
  if (name.trim().length === 0) {
    name = null;
  }
  dcfg.alias = name;
  cfg.setDevCfg(uuid, dcfg);
  dev.updateConfig(uuid, dcfg);
  res.sendStatus(200);
}

//-----------------------------------------------------------------------------

function onDevSave(req, res) {
  var uuid = req.query['uuid'];
  var dcfg = req.body;
  if (!uuid) {
    return res.status(400).send('Missing device UUID');
  }
  if (!dcfg || typeof dcfg !== 'object') {
    return res.status(400).send('Missing device config object');
  }
  cfg.setDevCfg(uuid, dcfg);
  dev.updateConfig(uuid, dcfg);
  res.sendStatus(200);
}

//-----------------------------------------------------------------------------

function onRefresh(req, res) {
  dev.refresh(function() {
    res.json(dev.getDevices());
  });
}

//-----------------------------------------------------------------------------

function onNotify(req, res) {
  srv.notifyAll(function() {
    res.sendStatus(200);
  });
}

//-----------------------------------------------------------------------------

function onXML(req, res) {
  var uuid = req.param('uuid');
  if (!uuid) {
    return res.status(400).send('Missing rename parameters');
  }
  var xml = dev.getXML(uuid);
  if (!xml) {
    return res.status(404).send('XML device desctiption not found');
  }
  res.set('Content-Type', 'text/plain; charset=utf-8');
  res.send(xml);
}

//-----------------------------------------------------------------------------

function saveConfig(res) {
  cfg.saveSelf(false, function(err) {
    if (err) {
      res.status(500).send(err.toString());
    }
    else {
      res.sendStatus(200);
    }
  });
}

//-----------------------------------------------------------------------------
// App Hooks
//-----------------------------------------------------------------------------

evt.onHttpAppReady(function(app) {
  cfg.rest(app).get('/ssdp/search', onSearch);
  cfg.rest(app).get('/ssdp/devs', onDevs);
  cfg.rest(app).get('/ssdp/block', onBlock);
  cfg.rest(app).get('/ssdp/hide', onHide);
  cfg.rest(app).get('/ssdp/rename', onRename);
  cfg.rest(app).get('/ssdp/devxml', onXML);
  cfg.rest(app).get('/ssdp/refresh', onRefresh);
  cfg.rest(app).get('/ssdp/notify', onNotify);
  cfg.rest(app).use('/ssdp/dsave', bodyParser.json());
  cfg.rest(app).use('/ssdp/dsave', onDevSave);
});
