/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var child = require('child_process');
var net   = require('net');
var env   = require('./env');

//-----------------------------------------------------------------------------
// killall
//-----------------------------------------------------------------------------

function killall_mac(procName, cb) {
  child.execFile('/usr/bin/killall', [procName], cb);
}

function killall_win(procName, cb) {
  child.exec('taskkill /F /IM ' + procName, cb);
}

exports.killall = env.ifMac(killall_mac, killall_win);

//-----------------------------------------------------------------------------
// freeport
//-----------------------------------------------------------------------------

function freeport(cb) {
  var server = net.createServer();
  var port   = 0;
  var error  = null;
  server.on('listening', function() {
    port = server.address().port;
    server.close()
  });
  server.on('error', function(err) {
    error = err;
  });
  server.on('close', function() {
    cb(error, port)
  });
  server.listen(0, '127.0.0.1')
}

exports.freeport = freeport;

//-----------------------------------------------------------------------------
// Ping
//-----------------------------------------------------------------------------

function ping_mac(addr, cb) {
  var cmd = '/sbin/ping';
  var arg = ['-c', '3', '-o', '-n', '-q', '-W', '1000', addr];
  child.execFile(cmd, arg, function(err) {
    cb(err);
  });
}

function ping_win(addr, cb) {
  var arg = ['ping', '-n', '3', '-w', '1000', addr];
  child.exec(arg.join(' '), function(err) {
    cb(err);
  });
}

exports.ping = env.ifMac(ping_mac, ping_win);