/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Power Management Plugin

'use strict';

var async = require('async');
var child = require('child_process');

var log = isedora['log'];
var evt = isedora['evt'];
var env = isedora['env'];
var cfg = isedora['cfg'];
var hnd = isedora['hnd'];

var didl = isedora['cls']['didl'];
var upnp = isedora['cls']['upnp'];

var Container = didl.Container;

var PLUGIN_NAME = 'isedora-pm';
var HANDLE_NAME = 'pm';

var TOP      = 'top';
var SLEEP    = 'sleep';
var RESTART  = 'restart';
var SHUTDOWN = 'shutdown';

var TOP_ID = mkoid(TOP);

var DEFS = {
  alias: {
    top:      'Power Management',
    sleep:    'Sleep',
    restart:  'Restart',
    shutdown: 'Shutdown'
  }
};

var enabled = isEnabled();

var pmproc = env.ifMac(pmosx, pmwin);

//-----------------------------------------------------------------------------

function getAlias(name) {
  return cfg.getPluginConfig(PLUGIN_NAME)['alias'][name];
}

//-----------------------------------------------------------------------------

hnd.addRootCount(function(brq) {
  return (enabled ? 1 : 0);
});


function rootHandler(brq, cb) {
  if (enabled && !brq.meta) {
    addRootOption(brq);
  }
  cb();
}

hnd.addRootBrowser(rootHandler);

//-----------------------------------------------------------------------------

function addRootOption(brq) {
  var opt = new Container(TOP_ID, hnd.ROOT, getAlias('top'), 3);
  opt.setHideCount(true);
  brq.ary.push(opt);
}

function addPowerOption(brq, oid, alias) {
  var opt = new Container(mkoid(oid), TOP_ID, getAlias(alias), 1);
  opt.setHideCount(true);
  brq.ary.push(opt);
}

//-----------------------------------------------------------------------------

function runProc(proc, arg) {
  var sary = [];
  evt.postGetServers(sary);
  async.each(sary, function(srv, cb) { srv.notifyByeBye(cb) }, function() { proc(arg) });
}

//-----------------------------------------------------------------------------

function pmHandle(brq, cb) {

  if (!enabled) return cb();

  var uid = brq.uid;

  if (brq.meta) {
    switch (uid) {
      case TOP:
        addRootOption(brq);
        break;
      case SLEEP:
        addPowerOption(brq, SLEEP, 'sleep');
        break;
      case RESTART:
        addPowerOption(brq, RESTART, 'restart');
        break;
      case SHUTDOWN:
        addPowerOption(brq, SHUTDOWN, 'shutdown');
        break;
    }
    return cb();
  }

  if (uid === TOP) {
    addPowerOption(brq, SLEEP, 'sleep');
    addPowerOption(brq, RESTART, 'restart');
    addPowerOption(brq, SHUTDOWN, 'shutdown');
    return cb();
  }

  log.info('Power command: %s', uid);

  runProc(pmproc, uid);

  cb();

}

hnd.addBrowseHandler(HANDLE_NAME, pmHandle);

//-----------------------------------------------------------------------------

function mkoid(name) {
  return hnd.mkoid(HANDLE_NAME, name);
}

//-----------------------------------------------------------------------------

function onConfigChange() {
  enabled = isEnabled();
}

evt.onConfigChange(onConfigChange);

//-----------------------------------------------------------------------------

function isEnabled() {
  return cfg.pluginEnabled(PLUGIN_NAME);
}

//-----------------------------------------------------------------------------

if (!cfg.getPluginConfig(PLUGIN_NAME)) {
  cfg.setPluginConfig(PLUGIN_NAME, DEFS);
}

//-----------------------------------------------------------------------------

function pmosx(action) {
  if (action === SHUTDOWN) {
    action = "shut down";
  }
  else {
    action = action.toLowerCase();
  }
  var script = 'tell application "System Events" to ' + action;
  child.execFile('/usr/bin/osascript', ['-e', script]);
}


function pmwin(action) {
  var arg;
  switch (action) {
    case SLEEP:
      arg = ['/h'];
      break;
    case RESTART:
      arg = ['/r', '/f', '/t', '0', '/d', 'p:0:0'];
      break;
    case SHUTDOWN:
      arg = ['/s', '/f', '/t', '0', '/d', 'p:0:0'];
  }
  child.exec('shutdown ' + arg.join(' '));
}




