/*
 * Copyright (c) 2015 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Early process exception setup

//require('/usr/local/lib/node_modules/v8-profiler');

process.env.UV_THREADPOOL_SIZE = 8;

var STDENC = (process.argv.indexOf('-stdutf16') > -1) ? 'utf16le' : 'utf8';

function processError(err) {
  if (!err) err = 'Unknown process exeption';
  try {
    process.stderr.write(err.toString() + "\n", STDENC, function() {
      process.abort();
    });
  }
  catch (e) {
    process.abort();
  }
}

//process.on('uncaughtException', processError);

// Start

process.env.NODE_ENV = 'production';

global.ISEDORA_DIR = __dirname;
global.ISEDORA_LIB = __dirname + '/lib';
global.ISEDORA_PKG = __dirname + '/package.json';


var path = require('path');
var fs   = require('fs');
var os   = require('os');

var lib = require('auto-load')(ISEDORA_LIB);

var evt = lib.evt;
var env = lib.env;
var agt = lib.agt;
var app = lib.app;
var cfg = lib.cfg;
var log = lib.log;
var ver = require(ISEDORA_PKG);

global.isedora = lib; // for plugins

var SIGINT  = 'SIGINT';
var SIGHUP  = 'SIGHUP';
var SIGTERM = 'SIGTERM';

var WATCH_TIMEOUT = 20 * 1000; // 20 seconds

function main() {
  startWatch();
  initSignals();
  evt.postInit();
  log.all("Starting %s %s", ver.description, ver.version);
  log.all("Runtime %s (%s) %s", process.version, process.arch, process.platform);
  agt.start();
}

function quit() {
  stopWatch();
  evt.quit();
}

function initSignals() {

  process.on(SIGINT, function() {
    log.info(SIGINT);
    quit()
  });
  process.on(SIGHUP, function() {
    log.info(SIGHUP);
    quit()
  });
  if (env.isMac()) {
    process.on(SIGTERM, function() {
      log.info(SIGTERM);
      quit()
    });
  }

  if (env.isWin()) {
    process.stdin.setEncoding(STDENC);
    process.stdin.on('close', function() {
      log.info('STDIN close');
      quit();
    });
    process.stdin.on('data', function(data) {
      log.trace('STDIN: %s', data);
    });
    process.stdin.on('end', function() {
      log.info('STDIN end');
      quit();
    });
  }

}

function writePid() {
  var lines = [];
  lines.push("httpPort = " + cfg.httpPort());
  lines.push("logFile = " + cfg.logFilePath());
  lines.push("userName = " + cfg.logonName());
  lines.push("");
  fs.writeFile(cfg.pidFilePath(), lines.join(os.EOL), {encoding: "utf8"}, function(err) {
    if (err) {
      log.error("Error writing pid file: %s", err);
      evt.throwGlobal('Error writing pid file');
    }
    else {
      evt.addExitProc(function(cb) {
        fs.unlink(cfg.pidFilePath(), function() { cb() });
      });
      log.debug("PID file written: %s", cfg.pidFilePath());
      evt.postPidFileReady();
    }
  });
}

// start watcher --------------------------------------------------------------

var watchTimer = null;

function startWatch() {
  watchTimer = setTimeout(
    function() { processError('Server startup takes too long') },
    WATCH_TIMEOUT
  );
}

function stopWatch() {
  if (watchTimer) {
    clearTimeout(watchTimer);
    watchTimer = null;
  }
}

// events ---------------------------------------------------------------------

evt.onHttpSrvReady(function(address) {
  writePid();
});

evt.onConfigReady(function() {
  setImmediate(app.start);
});

evt.onPidFileReady(function() {
  stopWatch();
  log.info("Broadcast server ready on port: %d", cfg.httpPort());
  agt.broadcastReady();
});

//-----------------------------------------------------------------------------

main();